/*
 * Copyright 2013 Mark Slater
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package net.sourceforge.writexml;

import static net.sourceforge.writexml.ObjectUtils.checkNotNull;

/**
 * An XML attribute.  For example, in {@code <tag name="value"/>}, {@code name="value"} is an attribute.
 * <p/>
 * Immutable and threadsafe.
 */
public final class Attribute {
    private final AttributeName name;
    private final AttributeValue value;
    private final Namespace namespace;

    private Attribute(final AttributeName name, final AttributeValue value, final Namespace namespace) {
        this.name = checkNotNull(name, "Cannot instantiate Attribute with null name");
        this.value = checkNotNull(value, "Cannot instantiate Attribute with null value");
        this.namespace = checkNotNull(namespace, "Cannot instantiate Attribute with null namespace");
    }

    /**
     * Factory method for constructing {@code Attribute}s.
     *
     * @param name      the name of the attribute.
     * @param value     the value of the attribute.
     * @param namespace the namespace of the attribute.
     * @return an {@code Attribute} representing an XML attribute with the given name, value, and namespace.
     */
    public static Attribute attribute(final AttributeName name, final AttributeValue value, final Namespace namespace) {
        return new Attribute(name, value, namespace);
    }

    AttributeName name() {
        return name;
    }

    AttributeValue value() {
        return value;
    }

    Namespace namespace() {
        return namespace;
    }

    @Override
    public boolean equals(final Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final Attribute attribute = (Attribute) o;
        return name.equals(attribute.name) && namespace.equals(attribute.namespace) && value.equals(attribute.value);

    }

    @Override
    public int hashCode() {
        int result = name.hashCode();
        result = 31 * result + value.hashCode();
        result = 31 * result + namespace.hashCode();
        return result;
    }

    @Override
    public String toString() {
        return "Attribute{" +
                "name=" + name +
                ", value=" + value +
                ", namespace=" + namespace +
                '}';
    }

}
