/*
 * Copyright 2013 Mark Slater
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package net.sourceforge.writexml;

import static net.sourceforge.writexml.ObjectUtils.checkNotNull;

/**
 * An XML document type definition.
 * <p/>
 * Immutable and threadsafe.
 */
public final class Dtd {
    private final DtdName name;
    private final DtdPublicId publicId;
    private final DtdSystemId systemId;

    private Dtd(final DtdName name, final DtdPublicId publicId, final DtdSystemId systemId) {
        this.name = checkNotNull(name, "Cannot instantiate Dtd with null name");
        this.publicId = checkNotNull(publicId, "Cannot instantiate Dtd with null public id");
        this.systemId = checkNotNull(systemId, "Cannot instantiate Dtd with null system id");
    }

    /**
     * Factory method for constructing {@code Dtd}s.
     *
     * @param name     the name of the DTD.
     * @param publicId the public identifier of the DTD.
     * @param systemId the system identifier of the DTD.
     * @return a {@code Dtd} representing an XML DTD with the given name, public identifier, and system identifier.
     */
    public static Dtd dtd(final DtdName name, final DtdPublicId publicId, final DtdSystemId systemId) {
        return new Dtd(name, publicId, systemId);
    }

    void writeTo(final XmlWriter xmlWriter) throws XmlWriteException {
        xmlWriter.writeDtd(name, publicId, systemId);
    }

    @Override
    public boolean equals(final Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final Dtd dtd = (Dtd) o;
        return name.equals(dtd.name) && publicId.equals(dtd.publicId) && systemId.equals(dtd.systemId);
    }

    @Override
    public int hashCode() {
        int result = name.hashCode();
        result = 31 * result + publicId.hashCode();
        result = 31 * result + systemId.hashCode();
        return result;
    }

    @Override
    public String toString() {
        return "Dtd{" +
                "name=" + name +
                ", publicId=" + publicId +
                ", systemId=" + systemId +
                '}';
    }
}
