/*
 * Copyright 2013 Mark Slater
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

package net.sourceforge.writexml;

import static java.util.Arrays.asList;
import static net.sourceforge.writexml.CollectionUtils.copy;
import static net.sourceforge.writexml.ObjectUtils.checkNotNull;

/**
 * An XML tag.  Tags can be empty or can contain text, cdata, other tags, or any combination of these.
 * <p/>
 * Immutable and threadsafe.
 */
public final class Tag extends TagContent {

    private final Namespace namespace;
    private final TagName name;
    private final Attributes attributes;
    private final Iterable<TagContent> children;

    private Tag(final Namespace namespace, final TagName name, final Attributes attributes, final Iterable<TagContent> children) {
        this.namespace = checkNotNull(namespace, "Cannot instantiate Tag with null namespace");
        this.name = checkNotNull(name, "Cannot instantiate Tag with null name");
        this.attributes = checkNotNull(attributes, "Cannot instantiate Tag with null attributes");
        this.children = copy(children);
    }

    /**
     * Factory method for making a {@code Tag}.
     *
     * @param namespace  the namespace of the tag.
     * @param name       the name of the tag.
     * @param attributes the attributes of the tag.
     * @param children   the children of the tag, if any.
     * @return a {@code Tag} representing the given parameters.
     */
    public static Tag tag(final Namespace namespace, final TagName name, final Attributes attributes, final TagContent... children) {
        return new Tag(namespace, name, attributes, asList(children));
    }

    /**
     * Factory method for making a {@code Tag}.
     *
     * @param namespace  the namespace of the tag.
     * @param name       the name of the tag.
     * @param attributes the attributes of the tag.
     * @param children   the children of the tag.
     * @return a {@code Tag} representing the given parameters.
     */
    public static Tag tag(final Namespace namespace, final TagName name, final Attributes attributes, final Iterable<TagContent> children) {
        return new Tag(namespace, name, attributes, children);
    }

    void writeTo(final XmlWriter xmlWriter) throws XmlWriteException {
        xmlWriter.writeStartTag(namespace, name, attributes);
        for (final TagContent child : children) {
            child.writeTo(xmlWriter);
        }
        xmlWriter.writeEndTag(namespace, name);
    }

    @Override
    public String toString() {
        return "Tag{" +
                "namespace=" + namespace +
                ", name=" + name +
                ", attributes=" + attributes +
                ", children=" + children +
                '}';
    }
}
